import 'package:flutter/material.dart';
import '../../utils/editable-cell-widgets.dart';
import '../../models/grid-cell.dart';

class EditableCellWidget extends EditableGridCellsStateful {
  ///Cell to be edited.
  final Cell cell;

  ///Callback function that will be invoked after the cell edition is completed.
  ///
  ///It will only get called if the current cell value is valid.
  final void Function() onEditingCompleteCallback;

  EditableCellWidget({this.cell, this.onEditingCompleteCallback}) : super();

  ///Gets the current value of the cell.
  String getValue() => this.cell.textEditingController.text;

  @override
  State<EditableCellWidget> createState() => _EditableCellWidgetState();
}

class _EditableCellWidgetState extends State<EditableCellWidget> {
  bool _isValid = true;

  _initialize() {
    _isValid = widget.cell.validator(widget.cell.getValue());
  }

  @override
  Widget build(BuildContext context) {
    _initialize();

    return Container(
      alignment: Alignment.center,
      width: 60,
      child: TextField(
        controller: widget.cell.textEditingController,
        cursorColor: Theme.of(context).cursorColor,
        textAlign: TextAlign.center,
        style: TextStyle(color: _isValid ? Colors.black : Colors.red),
        decoration: InputDecoration(
          enabledBorder: InputBorder.none,
          focusedBorder: InputBorder.none,
        ),
        onEditingComplete: () {
          this.setState(() {
            _isValid = widget.cell.validator(widget.cell.getValue());
          });
          //Only executing callback if value is valid
          if (_isValid) widget.cell.onEditingCompleteCallback();
        },
      ),
    );
  }
}

class HeaderWidget extends EditableGridCellsStateless {
  ///Text that will be shown in a header-like format.
  final String label;

  HeaderWidget({this.label}) : super();

  @override
  Widget build(BuildContext context) => Container(
        alignment: Alignment.center,
        width: 60,
        child: Text(
          label,
          style: TextStyle(
              color: Colors.black,
              fontWeight: FontWeight.bold,
              fontStyle: FontStyle.italic),
        ),
      );
}

class FixedCellWidget extends EditableGridCellsStateful {
  ///Read-only cell.
  final Cell cell;

  FixedCellWidget({this.cell}) : super();

  ///Gets the current value of the cell.
  String getValue() => this.cell.textEditingController.text;

  @override
  State<FixedCellWidget> createState() => _FixedCellWidgetState();
}

class _FixedCellWidgetState extends State<FixedCellWidget> {
  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.bottomCenter,
      width: 60,
      child: TextField(
        controller: widget.cell.textEditingController,
        textAlign: TextAlign.center,
        style: TextStyle(
            fontSize: 10,
            fontWeight: FontWeight.w800,
            fontStyle: FontStyle.italic,
            color: Colors.green),
        readOnly: true,
        decoration: InputDecoration(border: InputBorder.none),
      ),
    );
  }
}
